/* clock.c generated by valac 0.31.1.19-d5143, the Vala compiler
 * generated from clock.vala, do not modify */

/*
 * Copyright (C) 2013  Paolo Borelli <pborelli@gnome.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>


#define CLOCKS_TYPE_PANEL_ID (clocks_panel_id_get_type ())

#define CLOCKS_TYPE_CLOCK (clocks_clock_get_type ())
#define CLOCKS_CLOCK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CLOCKS_TYPE_CLOCK, ClocksClock))
#define CLOCKS_IS_CLOCK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CLOCKS_TYPE_CLOCK))
#define CLOCKS_CLOCK_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), CLOCKS_TYPE_CLOCK, ClocksClockIface))

typedef struct _ClocksClock ClocksClock;
typedef struct _ClocksClockIface ClocksClockIface;

#define CLOCKS_TYPE_HEADER_BAR (clocks_header_bar_get_type ())
#define CLOCKS_HEADER_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CLOCKS_TYPE_HEADER_BAR, ClocksHeaderBar))
#define CLOCKS_HEADER_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CLOCKS_TYPE_HEADER_BAR, ClocksHeaderBarClass))
#define CLOCKS_IS_HEADER_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CLOCKS_TYPE_HEADER_BAR))
#define CLOCKS_IS_HEADER_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CLOCKS_TYPE_HEADER_BAR))
#define CLOCKS_HEADER_BAR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), CLOCKS_TYPE_HEADER_BAR, ClocksHeaderBarClass))

typedef struct _ClocksHeaderBar ClocksHeaderBar;
typedef struct _ClocksHeaderBarClass ClocksHeaderBarClass;

typedef enum  {
	CLOCKS_PANEL_ID_WORLD,
	CLOCKS_PANEL_ID_ALARM,
	CLOCKS_PANEL_ID_STOPWATCH,
	CLOCKS_PANEL_ID_TIMER
} ClocksPanelId;

struct _ClocksClockIface {
	GTypeInterface parent_iface;
	void (*activate_new) (ClocksClock* self);
	void (*activate_select_all) (ClocksClock* self);
	void (*activate_select_none) (ClocksClock* self);
	gboolean (*escape_pressed) (ClocksClock* self);
	void (*update_header_bar) (ClocksClock* self);
	const gchar* (*get_label) (ClocksClock* self);
	void (*set_label) (ClocksClock* self, const gchar* value);
	ClocksHeaderBar* (*get_header_bar) (ClocksClock* self);
	void (*set_header_bar) (ClocksClock* self, ClocksHeaderBar* value);
	ClocksPanelId (*get_panel_id) (ClocksClock* self);
	void (*set_panel_id) (ClocksClock* self, ClocksPanelId value);
};



GType clocks_panel_id_get_type (void) G_GNUC_CONST;
#define CLOCKS_N_PANELS 4
GType clocks_header_bar_get_type (void) G_GNUC_CONST;
GType clocks_clock_get_type (void) G_GNUC_CONST;
void clocks_clock_activate_new (ClocksClock* self);
static void clocks_clock_real_activate_new (ClocksClock* self);
void clocks_clock_activate_select_all (ClocksClock* self);
static void clocks_clock_real_activate_select_all (ClocksClock* self);
void clocks_clock_activate_select_none (ClocksClock* self);
static void clocks_clock_real_activate_select_none (ClocksClock* self);
gboolean clocks_clock_escape_pressed (ClocksClock* self);
static gboolean clocks_clock_real_escape_pressed (ClocksClock* self);
void clocks_clock_update_header_bar (ClocksClock* self);
static void clocks_clock_real_update_header_bar (ClocksClock* self);
const gchar* clocks_clock_get_label (ClocksClock* self);
void clocks_clock_set_label (ClocksClock* self, const gchar* value);
ClocksHeaderBar* clocks_clock_get_header_bar (ClocksClock* self);
void clocks_clock_set_header_bar (ClocksClock* self, ClocksHeaderBar* value);
ClocksPanelId clocks_clock_get_panel_id (ClocksClock* self);
void clocks_clock_set_panel_id (ClocksClock* self, ClocksPanelId value);


GType clocks_panel_id_get_type (void) {
	static volatile gsize clocks_panel_id_type_id__volatile = 0;
	if (g_once_init_enter (&clocks_panel_id_type_id__volatile)) {
		static const GEnumValue values[] = {{CLOCKS_PANEL_ID_WORLD, "CLOCKS_PANEL_ID_WORLD", "world"}, {CLOCKS_PANEL_ID_ALARM, "CLOCKS_PANEL_ID_ALARM", "alarm"}, {CLOCKS_PANEL_ID_STOPWATCH, "CLOCKS_PANEL_ID_STOPWATCH", "stopwatch"}, {CLOCKS_PANEL_ID_TIMER, "CLOCKS_PANEL_ID_TIMER", "timer"}, {0, NULL, NULL}};
		GType clocks_panel_id_type_id;
		clocks_panel_id_type_id = g_enum_register_static ("ClocksPanelId", values);
		g_once_init_leave (&clocks_panel_id_type_id__volatile, clocks_panel_id_type_id);
	}
	return clocks_panel_id_type_id__volatile;
}


static void clocks_clock_real_activate_new (ClocksClock* self) {
}


void clocks_clock_activate_new (ClocksClock* self) {
	g_return_if_fail (self != NULL);
	CLOCKS_CLOCK_GET_INTERFACE (self)->activate_new (self);
}


static void clocks_clock_real_activate_select_all (ClocksClock* self) {
}


void clocks_clock_activate_select_all (ClocksClock* self) {
	g_return_if_fail (self != NULL);
	CLOCKS_CLOCK_GET_INTERFACE (self)->activate_select_all (self);
}


static void clocks_clock_real_activate_select_none (ClocksClock* self) {
}


void clocks_clock_activate_select_none (ClocksClock* self) {
	g_return_if_fail (self != NULL);
	CLOCKS_CLOCK_GET_INTERFACE (self)->activate_select_none (self);
}


static gboolean clocks_clock_real_escape_pressed (ClocksClock* self) {
	gboolean result = FALSE;
	result = FALSE;
	return result;
}


gboolean clocks_clock_escape_pressed (ClocksClock* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return CLOCKS_CLOCK_GET_INTERFACE (self)->escape_pressed (self);
}


static void clocks_clock_real_update_header_bar (ClocksClock* self) {
}


void clocks_clock_update_header_bar (ClocksClock* self) {
	g_return_if_fail (self != NULL);
	CLOCKS_CLOCK_GET_INTERFACE (self)->update_header_bar (self);
}


const gchar* clocks_clock_get_label (ClocksClock* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return CLOCKS_CLOCK_GET_INTERFACE (self)->get_label (self);
}


void clocks_clock_set_label (ClocksClock* self, const gchar* value) {
	g_return_if_fail (self != NULL);
	CLOCKS_CLOCK_GET_INTERFACE (self)->set_label (self, value);
}


ClocksHeaderBar* clocks_clock_get_header_bar (ClocksClock* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return CLOCKS_CLOCK_GET_INTERFACE (self)->get_header_bar (self);
}


void clocks_clock_set_header_bar (ClocksClock* self, ClocksHeaderBar* value) {
	g_return_if_fail (self != NULL);
	CLOCKS_CLOCK_GET_INTERFACE (self)->set_header_bar (self, value);
}


ClocksPanelId clocks_clock_get_panel_id (ClocksClock* self) {
	g_return_val_if_fail (self != NULL, 0);
	return CLOCKS_CLOCK_GET_INTERFACE (self)->get_panel_id (self);
}


void clocks_clock_set_panel_id (ClocksClock* self, ClocksPanelId value) {
	g_return_if_fail (self != NULL);
	CLOCKS_CLOCK_GET_INTERFACE (self)->set_panel_id (self, value);
}


static void clocks_clock_base_init (ClocksClockIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
		g_object_interface_install_property (iface, g_param_spec_string ("label", "label", "label", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
		g_object_interface_install_property (iface, g_param_spec_object ("header-bar", "header-bar", "header-bar", CLOCKS_TYPE_HEADER_BAR, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
		g_object_interface_install_property (iface, g_param_spec_enum ("panel-id", "panel-id", "panel-id", CLOCKS_TYPE_PANEL_ID, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
		iface->activate_new = clocks_clock_real_activate_new;
		iface->activate_select_all = clocks_clock_real_activate_select_all;
		iface->activate_select_none = clocks_clock_real_activate_select_none;
		iface->escape_pressed = clocks_clock_real_escape_pressed;
		iface->update_header_bar = clocks_clock_real_update_header_bar;
	}
}


GType clocks_clock_get_type (void) {
	static volatile gsize clocks_clock_type_id__volatile = 0;
	if (g_once_init_enter (&clocks_clock_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ClocksClockIface), (GBaseInitFunc) clocks_clock_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		GType clocks_clock_type_id;
		clocks_clock_type_id = g_type_register_static (G_TYPE_INTERFACE, "ClocksClock", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (clocks_clock_type_id, G_TYPE_OBJECT);
		g_once_init_leave (&clocks_clock_type_id__volatile, clocks_clock_type_id);
	}
	return clocks_clock_type_id__volatile;
}



